<?php

namespace Drupal\couchbasedrupal;

use Drupal\Core\DependencyInjection\ServiceModifierInterface;
use Drupal\Core\DependencyInjection\ServiceProviderInterface;
use Drupal\Core\DependencyInjection\ContainerBuilder;

use Drupal\Core\Site\Settings;

use Drupal\couchbasedrupal\Cache\CouchbaseBackendFactory;
use Drupal\couchbasedrupal\Cache\CouchbaseTagsChecksum;

use Drupal\Core\Cache\DatabaseCacheTagsChecksum;

class CouchbasedrupalServiceProvider implements ServiceProviderInterface {
  /**
   * {@inheritdoc}
   */
  public function register(ContainerBuilder $container) {

    // Make sure we have a properly setup couchbase backend before overriding the services.
    $settings = Settings::get('couchbasedrupal');
    $cluster = CouchbaseBackendFactory::GetCouchbaseServer($settings);

    // If we could not find a valid cluster, do not override the services.
    if ($cluster === FALSE) {
      // If we cannot retrieve the couchbase server
      // do not modify the service container and log
      // a warning.
      \Drupal::logger('couchbasedrupal')->notice('No valid couchbase server was found in your site settings. Configure your server settings and rebuild the service container.');
      return;
    }


    // Set chained fast backend persitent storage
    $definition  = $container->getDefinition('cache.backend.chainedfast');
    $args = $definition->getArguments();
    // Add the missins optional arguments
    if (!isset($args[1])) {
      $args[] = 'cache.backend.couchbase';
      $definition->setArguments($args);
    }

    // Override tag checksum invalidator
    $definition = $container->getDefinition('cache_tags.invalidator.checksum');
    if ($definition->getClass() == DatabaseCacheTagsChecksum::class) {
      $definition->setClass(CouchbaseTagsChecksum::class);
      $definition->setArguments(array());
    }

    // Override locking backend
    // De momento deshabilitar, las ganancias de rendimiento son mínimas
    // y no está muy bien probado. Además necesita un lazy builder - proxy -
    // que no est´abien armado.
    //  $definition = $container->getDefinition('lock');
    //  if ($definition->getClass() == \Drupal\Core\Lock\DatabaseLockBackend::class) {
    //    $definition->setClass(\Drupal\couchbase\Lock\CouchbaseLockBackend::class);
    //    $definition->setArguments(array());
    //  }

  }
}
